import { Show, For, splitProps, mergeProps, createEffect, on, createContext, useContext, } from "solid-js";
import { createSelect, } from "./create-select";
const SelectContext = createContext();
const useSelect = () => {
    const context = useContext(SelectContext);
    if (!context)
        throw new Error("No SelectContext found in ancestry.");
    return context;
};
const Select = (props) => {
    const [selectProps, local] = splitProps(mergeProps({
        format: ((data, type) => data),
        placeholder: "Auswählen...",
        readonly: typeof props.options !== "function",
        loading: false,
        loadingPlaceholder: "Laden...",
        emptyPlaceholder: "Keine Auswahl...",
    }, props), [
        "options",
        "optionToValue",
        "isOptionDisabled",
        "multiple",
        "disabled",
        "onInput",
        "onChange",
        "unique",
    ]);
    const select = createSelect(selectProps);
    createEffect(on(() => local.initialValue, (value) => value !== undefined && select.setValue(value)));
    return (<SelectContext.Provider value={select}>
      <Container class={local.class}>
        <Control type={local.type} id={local.id} name={local.name} format={local.format} placeholder={local.placeholder} autofocus={local.autofocus} readonly={local.readonly}/>
        <List loading={local.loading} loadingPlaceholder={local.loadingPlaceholder} emptyPlaceholder={local.emptyPlaceholder} format={local.format}/>
      </Container>
    </SelectContext.Provider>);
};
const Container = (props) => {
    const select = useSelect();
    return (<div class={`solid-select-container ${props.class !== undefined ? props.class : ""}`} data-disabled={select.disabled} onFocusIn={select.onFocusIn} onFocusOut={select.onFocusOut} onMouseDown={(event) => {
            select.onMouseDown(event);
            event.currentTarget.getElementsByTagName("input")[0].focus();
        }}>
      {props.children}
    </div>);
};
const Control = (props) => {
    const select = useSelect();
    const removeValue = (index) => {
        const value = select.value();
        select.setValue([...value.slice(0, index), ...value.slice(index + 1)]);
    };
    return (<div class="solid-select-control bg-primary-subtle" data-multiple={select.multiple} data-has-value={select.hasValue()} data-disabled={select.disabled} onClick={select.onClick}>
      <Show when={!select.hasValue() && !select.hasInputValue()}>
        <Placeholder>{props.placeholder}</Placeholder>
      </Show>
      <Show when={select.hasValue() && !select.multiple && !select.hasInputValue()}>
        <SingleValue>{props.format(select.value(), "value")}</SingleValue>
      </Show>
      <Show when={select.hasValue() && select.multiple}>
        <For each={select.value()}>
          {(value, index) => (<MultiValue disabled={select.disabled} onRemove={() => removeValue(index())}>
              {props.format(value, "value")}
            </MultiValue>)}
        </For>
      </Show>
      <Input id={props.id} name={props.name} autofocus={props.autofocus} readonly={props.readonly} type={props.type}/>
    </div>);
};
const Placeholder = (props) => {
    return <div class="solid-select-placeholder">{props.children}</div>;
};
const SingleValue = (props) => {
    return <div class="solid-select-single-value">{props.children}</div>;
};
const MultiValue = (props) => {
    const select = useSelect();
    return (<div class="solid-select-multi-value bg-primary">
      {props.children}
      <Show when={!props.disabled}>
        <button type="button" class="solid-select-multi-value-remove btn" onClick={(event) => {
            event.stopPropagation();
            props.onRemove();
        }}>
          ⨯
        </button>
      </Show>
    </div>);
};
const Input = (props) => {
    const select = useSelect();
    return (<input id={props.id} name={props.name} class="solid-select-input" data-multiple={select.multiple} data-is-active={select.isActive()} type={props.type} tabIndex={0} autocomplete="off" autocapitalize="none" autofocus={props.autofocus} readonly={props.readonly} disabled={select.disabled} size={1} value={select.inputValue()} onInput={select.onInput} onKeyDown={(event) => {
            select.onKeyDown(event);
            if (!event.defaultPrevented) {
                if (event.key === "Escape") {
                    event.preventDefault();
                    event.stopPropagation();
                    event.target.blur();
                }
            }
        }} onMouseDown={(event) => {
            event.stopPropagation();
        }}/>);
};
const List = (props) => {
    const select = useSelect();
    return (<Show when={select.isOpen()}>
      <div class="solid-select-list list-group">
        <Show when={!props.loading} fallback={<div class="solid-select-list-placeholder list-group-item">
              {props.loadingPlaceholder}
            </div>}>
          <For each={select.options()} fallback={<div class="solid-select-list-placeholder list-group-item">
                {props.emptyPlaceholder}
              </div>}>
            {(option) => {
            if (!select.value().includes(option.value) && select.unique) {
                return (<Option option={option}>
                    {props.format(option, "option")}
                  </Option>);
            }
            else if (!select.unique) {
                return (<Option option={option}>
                    {props.format(option, "option")}
                  </Option>);
            }
        }}
          </For>
        </Show>
      </div>
    </Show>);
};
const Option = (props) => {
    const select = useSelect();
    const scrollIntoViewOnFocus = (element) => {
        createEffect(() => {
            if (select.isOptionFocused(props.option)) {
                element.scrollIntoView({ block: "nearest" });
            }
        });
    };
    return (<div ref={scrollIntoViewOnFocus} data-disabled={select.isOptionDisabled(props.option)} data-focused={select.isOptionFocused(props.option)} class="solid-select-option list-group-item" onClick={() => select.pickOption(props.option)}>
      {props.children}
    </div>);
};
export { Select, Container, Control, Placeholder, SingleValue, MultiValue, Input, List, Option, SelectContext, useSelect, };
